#region Copyright
/*
 * Copyright 2005-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System;
using Seasar.Quill.Attrs;
using System.Reflection;
using System.Collections.Generic;

namespace Seasar.Quill.Util
{
    /// <summary>
    /// QuillŗpӂĂ鑮NX
    /// </summary>
    public static class AttributeUtil
    {
        #region ImplementationAttribute

        /// <summary>
        /// NXw肷ׂɐݒ肳Ă鑮
        /// (<see cref="Seasar.Quill.Attrs.ImplementationAttribute"/>)擾
        /// </summary>
        /// <param name="type">mFType</param>
        /// <returns>NXw肳ꂽ</returns>
        public static ImplementationAttribute GetImplementationAttr(Type type)
        {
            // NXw肷鑮擾
            ImplementationAttribute implAttr =
                (ImplementationAttribute)Attribute.GetCustomAttribute(
                type, typeof(ImplementationAttribute));

            if(implAttr == null)
            {
                // Implementationw肳ĂȂꍇnullԂ
                return null;
            }

            // ImplementationɎw肳ꂽNXType
            Type implType = implAttr.ImplementationType;

            if (!type.IsInterface && implType != null)
            {
                // NXImplementationɎNXw肳Ăꍇ
                // OX[
                throw new QuillApplicationException("EQLL0001",
                    new object[] { type.FullName });
            }

            if (implType != null && implType.IsInterface)
            {
                // Implementation̎NXɃC^[tF[X
                // w肳Ăꍇ͗OX[
                throw new QuillApplicationException("EQLL0002", new object[] {
                    type.FullName, implType.FullName });
            }

            if (implType != null && implType.IsAbstract)
            {
                // Implementation̎NXɒۃNX
                // w肳Ăꍇ͗OX[
                throw new QuillApplicationException("EQLL0003", new object[] {
                    type.FullName, implType.FullName });
            }

            if (implType != null && !type.IsAssignableFrom(implType))
            {
                // s\ȎNXw肳Ăꍇ͗OX[
                throw new QuillApplicationException("EQLL0004", new object[] {
                    type.FullName, implType.FullName });
            }

            // NXw肷鑮Ԃ
            return implAttr;
        }

        #endregion

        #region AspectAttribute

        /// <summary>
        /// Aspectw肷ׂɃNXC^[tF[Xɐݒ肳Ă鑮
        /// (<see cref="Seasar.Quill.Attrs.AspectAttribute"/>)擾
        /// </summary>
        /// <param name="type">mFType</param>
        /// <returns>Aspectw肳ꂽ̔z</returns>
        public static AspectAttribute[] GetAspectAttrs(Type type)
        {
            if (!type.IsPublic && !type.IsNestedPublic)
            {
                // \bh錾NXpublicł͂Ȃꍇ͗OX[
                throw new QuillApplicationException("EQLL0016", new object[] {
                    type.FullName });
            }

            // Aspectw肷鑮擾ĕԂ
            return GetAspectAttrsByMember(type);
        }

        /// <summary>
        /// Aspectw肷ׂɃ\bhɐݒ肳Ă鑮
        /// (<see cref="Seasar.Quill.Attrs.AspectAttribute"/>)擾
        /// </summary>
        /// <param name="method">mF郁\bh</param>
        /// <returns>Aspectw肳ꂽ̔z</returns>
        public static AspectAttribute[] GetAspectAttrsByMethod(MethodInfo method)
        {
            // Aspectw肷鑮擾
            AspectAttribute[] attrs = GetAspectAttrsByMember(method);

            if (attrs.Length == 0)
            {
                // Aspectw肳ĂȂꍇ͔zTCY0Aspect̔zԂ
                return attrs;
            }

            if (!method.DeclaringType.IsPublic && !method.DeclaringType.IsNestedPublic)
            {
                // \bh錾NXpublicł͂Ȃꍇ͗OX[
                throw new QuillApplicationException("EQLL0016", new object[] {
                    method.DeclaringType.FullName });
            }

            if (method.IsStatic)
            {
                // \bhstatic̏ꍇ͗OX[
                throw new QuillApplicationException("EQLL0005", new object[] {
                    method.DeclaringType.FullName, method.Name });
            }

            if (!method.IsPublic)
            {
                // \bhpublicł͂Ȃꍇ͗OX[
                throw new QuillApplicationException("EQLL0006", new object[] {
                    method.DeclaringType.FullName, method.Name });
            }

            if (!method.IsVirtual)
            {
                // \bhvirtualC^[tF[X̃\bh
                // ł͂Ȃꍇ͗OX[
                throw new QuillApplicationException("EQLL0007", new object[] {
                    method.DeclaringType.FullName, method.Name });
            }

            // Aspectw肷鑮Ԃ
            return attrs;
        }

        /// <summary>
        /// Aspectw肷ׂɃoɐݒ肳Ă鑮
        /// (<see cref="Seasar.Quill.Attrs.AspectAttribute"/>)擾
        /// </summary>
        /// <param name="member">mF郁o</param>
        /// <returns>Aspectw肳ꂽ̔z</returns>
        public static AspectAttribute[] GetAspectAttrsByMember(MemberInfo member)
        {
            // Aspectw肷鑮擾
            AspectAttribute[] attrs =
                (AspectAttribute[])Attribute.GetCustomAttributes(
                member, typeof(AspectAttribute));

            // Aspecti[郊Xg
            List<AspectAttribute> attrList = 
                new List<AspectAttribute>(attrs);

            // Aspect̃Xgёւ
            attrList.Sort(new AspectAttributeComparer());

            // Aspectw肷鑮Ԃ
            return attrList.ToArray();
        }

        #endregion

        #region BindingAttribute

        /// <summary>
        /// S2Container̃R|[lgBindingw肷ׂɃtB[hɐݒ肳Ă
        /// (<see cref="Seasar.Quill.Attrs.BindingAttribute"/>)擾
        /// </summary>
        /// <param name="field">tB[h</param>
        /// <returns>BindingR|[lgw肳ꂽBinding</returns>
        public static BindingAttribute GetBindingAttr(FieldInfo field)
        {
            if (field.IsStatic)
            {
                // statictB[h̏ꍇ͗OX[
                throw new QuillApplicationException("EQLL0015", new string[] {
                    field.DeclaringType.FullName, field.Name});
            }

            // oCfBOR|[lgw肷鑮擾
            BindingAttribute bindingAttr =
                (BindingAttribute)Attribute.GetCustomAttribute(
                field, typeof(BindingAttribute));

            // oCfBOw肳ĂȂꍇnullԂ
            if (bindingAttr == null || bindingAttr.ComponentName == null)
            {
                return null;
            }

            // BindingԂ
            return bindingAttr;
        }

        #endregion

        #region AspectAttributerNX

        /// <summary>
        /// AspectAttributerNX
        /// </summary>
        private class AspectAttributeComparer : IComparer<AspectAttribute>
        {
            /// <summary>
            /// 2AspectAttributer
            /// iя<see cref="Seasar.Quill.Attrs.AspectAttribute.Ordinal"/>
            /// 肷)
            /// <para>
            /// xyꍇ0, xy傫ꍇ͐̒l,
            /// xy菬ꍇ͕̒lԂ
            /// </para>
            /// </summary>
            /// <param name="x">rΏۂ̑1IuWFNg</param>
            /// <param name="y">rΏۂ̑2IuWFNg</param>
            /// <returns>xyꍇ0, xy傫ꍇ͐̒l,
            /// xy菬ꍇ͕̒lԂ</returns>
            public int Compare(AspectAttribute x, AspectAttribute y)
            {
                // xyꍇ0, xy傫ꍇ͐̒l,
                // xy菬ꍇ͕̒lԂ
                return x.Ordinal - y.Ordinal;
            }
        }

        #endregion
    }
}
